@@ -35,11 +35,12 @@ class Response(httplib.HTTPResponse):
         return httplib.OK <= self.status < httplib.MULTIPLE_CHOICES
     def __call__(self):
         "Return evaluated response body or raise exception."
-        if not self:
-            raise httplib.HTTPException(self.status, self.reason, self.body)
-        if self.body and self.getheader('content-type') == 'text/x-json':
-            return json.loads(self.body)
-        return self.body
+        body = self.body
+        if body and self.getheader('content-type') == 'text/x-json':
+            body = json.loads(body)
+        if self:
+            return body
+        raise httplib.HTTPException(self.status, self.reason, body)
 
 class Resource(httplib.HTTPConnection):
     "Synchronous connection which handles json responses."
@@ -58,7 +59,7 @@ class Resource(httplib.HTTPConnection):
         return self.getresponse()()
     def get(self, path, **params):
         if params:
-            path += '?' + urllib.urlencode(params)
+            path += '?' + urllib.urlencode(params, doseq=True)
         return self('GET', path)
     def post(self, path, **body):
         return self('POST', path, body)
@@ -66,7 +67,7 @@ class Resource(httplib.HTTPConnection):
         return self('PUT', path, body)
     def delete(self, path, **params):
         if params:
-            path += '?' + urllib.urlencode(params)
+            path += '?' + urllib.urlencode(params, doseq=True)
         return self('DELETE', path)
 
 class Resources(dict):
@@ -98,10 +99,11 @@ class Resources(dict):
         return -len(self[host])
     def choice(self, hosts):
         "Return chosen host according to priority."
-        hosts = dict(zip(hosts, map(self.priority, hosts)))
-        for priority, candidates in itertools.groupby(sorted(hosts, key=hosts.get), hosts.get):
-            if priority is not None:
-                return random.choice(list(candidates))
+        priorities = collections.defaultdict(list)
+        for host in hosts:
+            priorities[self.priority(host)].append(host)
+        priorities.pop(None, None)
+        return random.choice(priorities[min(priorities)])
     def unicast(self, method, path, body=None, hosts=()):
         "Send request and return response from any host, optionally from given subset."
         host = self.choice(tuple(hosts) or self)
