@@ -9,7 +9,7 @@ import contextlib
 import abc, collections
 import warnings
 import lucene
-from .queries import Query, HitCollector, Filter
+from .queries import Query, HitCollector, Filter, Highlighter
 from .documents import Field, Document, Hits
 
 class Atomic(object):
@@ -21,6 +21,12 @@ class Atomic(object):
 Atomic.register(basestring)
 Atomic.register(lucene.TokenStream)
 
+class closing(set):
+    "Manage lifespan of registered objects, similar to contextlib.closing."
+    def __del__(self):
+        for obj in self:
+            obj.close()
+
 class TokenFilter(lucene.PythonTokenFilter):
     """Create an iterable lucene TokenFilter from a TokenStream.
     In lucene version 2, call on any iterable of tokens.
@@ -59,11 +65,11 @@ class TokenFilter(lucene.PythonTokenFilter):
     @property
     def payload(self):
         "Payload bytes."
-        data = self.Payload.payload.data
-        return data and data.string_
+        payload = self.Payload.payload
+        return payload and getattr(payload.data, 'string_', None)
     @payload.setter
     def payload(self, data):
-        self.Payload.payload = lucene.Payload(lucene.JArray_byte(bytes(data)))
+        self.Payload.payload = lucene.Payload(lucene.JArray_byte(data))
     @property
     def positionIncrement(self):
         "Position relative to the previous token."
@@ -100,24 +106,30 @@ class Analyzer(lucene.PythonAnalyzer):
         for filter in self.filters:
             tokens = filter(tokens)
         return tokens if isinstance(tokens, lucene.TokenStream) else TokenFilter(tokens)
-    def tokens(self, text):
+    def tokens(self, text, field=None):
         "Return lucene TokenStream from text."
-        return self.tokenStream('', lucene.StringReader(text))
+        return self.tokenStream(field, lucene.StringReader(text))
     def parse(self, query, field='', op='', version='current', **attrs):
         """Return parsed lucene Query.
         
-        :param field: default query field name or sequence of names
+        :param query: query string
+        :param field: default query field name, sequence of names, or (in lucene 2.9 or higher) boost mapping
         :param op: default query operator ('or', 'and')
         :param version: lucene Version string, leave blank for deprecated constructor
         :param attrs: additional attributes to set on the parser
         """
         # parsers aren't thread-safe (nor slow), so create one each time
-        parser = lucene.QueryParser if isinstance(field, basestring) else lucene.MultiFieldQueryParser
-        version = 'LUCENE_' + version.replace('.', '').upper()
-        try:
-            parser = parser(getattr(lucene.Version, version), field, self)
-        except (AttributeError, lucene.InvalidArgsError):
-            parser = parser(field, self) # 2.4 lacks Version, 2.9.0 lacks constructor
+        args = []
+        if version and lucene.VERSION >= '2.9.1':
+            args += getattr(lucene.Version, 'LUCENE_' + version.replace('.', '').upper()),
+        if isinstance(field, collections.Mapping):
+            boosts = lucene.HashMap()
+            for key in field:
+                boosts.put(key, lucene.Float(field[key]))
+            args += list(field), self, boosts
+        else:
+            args += field, self
+        parser = (lucene.QueryParser if isinstance(field, basestring) else lucene.MultiFieldQueryParser)(*args)
         if op:
             parser.defaultOperator = getattr(lucene.QueryParser.Operator, op.upper())
         for name, value in attrs.items():
@@ -263,47 +275,58 @@ class IndexReader(object):
 
 class Searcher(object):
     "Mixin interface common among searchers."
-    class StandardAnalyzer(lucene.StandardAnalyzer):
-        __del__ = lucene.StandardAnalyzer.close
     def __init__(self, arg, analyzer=None):
         super(Searcher, self).__init__(arg)
         if analyzer is None:
-            analyzer = self.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else self.StandardAnalyzer()
+            analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else lucene.StandardAnalyzer()
+            self.shared.add(analyzer)
         self.analyzer = analyzer
+        self.filters = {}
     def __del__(self):
-        if str(self) != '<null>':
+        if hash(self):
             self.close()
+    def open(self, *directories):
+        self.shared = closing()
+        for directory in directories:
+            if isinstance(directory, basestring):
+                if hasattr(lucene.FSDirectory, 'open'):
+                    directory = lucene.FSDirectory.open(lucene.File(directory))
+                else:
+                    directory = lucene.FSDirectory.getDirectory(directory)
+                self.shared.add(directory)
+            yield directory
+    def reopen(self):
+        "Return current Searcher, only creating a new one if necessary."
+        reader = self.indexReader.reopen()
+        if reader == self.indexReader:
+            return self
+        other = type(self)(reader, self.analyzer)
+        other.shared, other.owned = self.shared, closing([reader])
+        return other
     def __getitem__(self, id):
         return Document(self.doc(id))
     def parse(self, query, *args, **kwargs):
         if isinstance(query, lucene.Query):
             return query
         return Analyzer.parse.im_func(self.analyzer, query, *args, **kwargs)
-    def highlight(self, query, text, count=1, span=True, formatter=None, encoder=None, field='', **attrs):
-        """Return highlighted text fragments which match the query.
+    def highlighter(self, query, span=True, formatter=None, encoder=None, field=None):
+        "Return `Highlighter`_ specific to the searcher's analyzer and index."
+        query = self.parse(query, field or '')
+        return Highlighter(query, self.analyzer, span, formatter, encoder, field, (field and self.indexReader))
+    def highlight(self, query, doc, count=1, span=True, formatter=None, encoder=None, field=None, **attrs):
+        """Return highlighted text fragments which match the query, using internal :meth:`highlighter`.
         
         :param query: query string or lucene Query
-        :param text: text string to be searched
+        :param doc: text string or doc id to be highlighted
         :param count: maximum number of fragments
-        :param span: only highlight terms which would contribute to a hit
-        :param formatter: optional lucene Formatter
-        :param encoder: optional lucene Encoder
-        :param field: default query field name
         :param attrs: additional attributes to set on the highlighter
         """
-        query = self.parse(query, field)
-        highlighter = lucene.Highlighter(*filter(None, [formatter, encoder, lucene.QueryScorer(query, field)]))
+        highlighter = self.highlighter(query, span, formatter, encoder, field)
         for name, value in attrs.items():
             setattr(highlighter, name, value)
-        if lucene.VERSION <= '2.4.1': # memory leak in string array
-            tokens = self.analyzer.tokenStream(field, lucene.StringReader(text))
-            if span:
-                tokens = lucene.CachingTokenFilter(tokens)
-                highlighter.fragmentScorer = lucene.HighlighterSpanScorer(query, field, tokens)
-            return map(unicode, highlighter.getBestTextFragments(tokens, text, True, count))
-        if not span:
-            highlighter.fragmentScorer = lucene.QueryTermScorer(query, field)
-        return list(highlighter.getBestFragments(self.analyzer, field, text, count))
+        if not isinstance(doc, basestring):
+            doc = self[doc][field]
+        return highlighter.fragments(doc, count)
     def count(self, *query, **options):
         """Return number of hits for given query or term.
         
@@ -342,6 +365,29 @@ class Searcher(object):
         scoredocs = list(topdocs.scoreDocs)
         ids, scores = (map(operator.attrgetter(name), scoredocs) for name in ('doc', 'score'))
         return Hits(self, ids, scores, topdocs.totalHits)
+    def facets(self, ids, *keys):
+        """Return mapping of document counts for the intersection with each facet.
+        
+        :param ids: document ids or lucene Filter
+        :param keys: field names, term tuples, or any keys to previously cached filters
+        """
+        counts = collections.defaultdict(dict)
+        if not isinstance(ids, lucene.Filter):
+            ids = Filter(ids)
+        for key in keys:
+            filters = self.filters.get(key)
+            if isinstance(filters, lucene.Filter):
+                counts[key] = Filter.overlap(ids, filters, self.indexReader)
+            elif isinstance(key, basestring):
+                values = self.terms(key) if filters is None else filters
+                counts.update(self.facets(ids, *((key, value) for value in values)))
+            else:
+                name, value = key
+                filters = self.filters.setdefault(name, {})
+                if value not in filters:
+                    filters[value] = Query.term(name, value).filter()
+                counts[name][value] = Filter.overlap(ids, filters[value], self.indexReader)
+        return dict(counts)
 
 class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
     """Inherited lucene IndexSearcher, with a mixed-in IndexReader.
@@ -350,51 +396,23 @@ class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
     :param analyzer: lucene Analyzer, default StandardAnalyzer
     """
     def __init__(self, directory, analyzer=None):
-        self.closing = set()
-        if isinstance(directory, basestring):
-            if hasattr(lucene.FSDirectory, 'open'):
-                directory = lucene.FSDirectory.open(lucene.File(directory))
-            else:
-                directory = lucene.FSDirectory.getDirectory(directory)
-            self.closing.add(directory)
-        Searcher.__init__(self, directory, analyzer)
-        self.filters = {}
-    def __del__(self):
-        if str(self) != '<null>':
-            self.close()
-        for obj in self.closing:
-            obj.close()
-    def facets(self, ids, *keys):
-        """Return mapping of document counts for the intersection with each facet.
-        
-        :param ids: document ids
-        :param keys: field names, term tuples, or any keys to previously cached filters
-        """
-        counts = collections.defaultdict(dict)
-        ids = Filter(ids)
-        for key in keys:
-            filters = self.filters.get(key)
-            if isinstance(filters, Filter):
-                counts[key] = ids.overlap(filters, self.indexReader)
-            elif isinstance(key, basestring):
-                values = self.terms(key) if filters is None else filters
-                counts.update(self.facets(ids.docIdSet, *((key, value) for value in values)))
-            else:
-                name, value = key
-                filters = self.filters.setdefault(name, {})
-                if value not in filters:
-                    filters[value] = Query.term(name, value).filter()
-                counts[name][value] = ids.overlap(filters[value], self.indexReader)
-        return dict(counts)
+        Searcher.__init__(self, *self.open(directory), analyzer=analyzer)
 
-class MultiSearcher(Searcher, lucene.MultiSearcher):
+class MultiSearcher(Searcher, lucene.MultiSearcher, IndexReader):
     """Inherited lucene MultiSearcher.
+    All sub searchers will be closed when the MultiSearcher is closed.
     
-    :param searchers: lucene.Searchers or directory
+    :param searchers: directory paths, Directories, IndexReaders, Searchers, or a MultiReader
     :param analyzer: lucene Analyzer, default StandardAnalyzer
     """
     def __init__(self, searchers, analyzer=None):
-        searchers = [searcher if isinstance(searcher, lucene.Searcher) else lucene.IndexSearcher(searcher) for searcher in searchers]
+        if lucene.MultiReader.instance_(searchers):
+            self.indexReader = searchers
+            searchers = map(lucene.IndexSearcher, searchers.sequentialSubReaders)
+        else:
+            searchers = [searcher if isinstance(searcher, lucene.Searcher) else lucene.IndexSearcher(searcher) for searcher in self.open(*searchers)]
+            self.indexReader = lucene.MultiReader(map(operator.attrgetter('indexReader'), searchers), False)
+            self.owned = closing([self.indexReader])
         Searcher.__init__(self, searchers, analyzer)
 
 class ParallelMultiSearcher(MultiSearcher, lucene.ParallelMultiSearcher):
@@ -412,10 +430,10 @@ class IndexWriter(lucene.IndexWriter):
     __del__ = IndexSearcher.__del__.im_func
     parse = Searcher.parse.im_func
     def __init__(self, directory=None, mode='a', analyzer=None):
-        self.closing = set()
+        self.shared = closing()
         if analyzer is None:
             analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else lucene.StandardAnalyzer()
-            self.closing.add(analyzer)
+            self.shared.add(analyzer)
         if not isinstance(directory, lucene.Directory):
             if directory is None:
                 directory = lucene.RAMDirectory()
@@ -423,12 +441,12 @@ class IndexWriter(lucene.IndexWriter):
                 directory = lucene.FSDirectory.open(lucene.File(directory))
             else:
                 directory = lucene.FSDirectory.getDirectory(directory)
-            self.closing.add(directory)
+            self.shared.add(directory)
         mfl = lucene.IndexWriter.MaxFieldLength.LIMITED
         if mode == 'a':
             lucene.IndexWriter.__init__(self, directory, analyzer, mfl)
         else:
-            lucene.IndexWriter.__init__(self, directory, analyzer, (mode == 'w'), mfl)
+            lucene.IndexWriter.__init__(self, directory, analyzer, bool('rw'.index(mode)), mfl)
         self.fields = {}
     @property
     def segments(self):
@@ -503,6 +521,4 @@ class Indexer(IndexWriter):
     def commit(self):
         "Commit writes and refresh searcher.  Not thread-safe."
         IndexWriter.commit(self)
-        if not self.current:
-            searcher = self.indexSearcher = IndexSearcher(self.reopen(), self.analyzer)
-            searcher.closing.add(searcher.indexReader)
+        self.indexSearcher = self.indexSearcher.reopen()
