@@ -32,23 +32,28 @@ class Query(object):
         return self
     @classmethod
     def any(cls, *queries, **terms):
-        "Return lucene BooleanQuery (OR) from queries and terms."
+        "Return :class:`BooleanQuery` (OR) from queries and terms."
         return cls.boolean(lucene.BooleanClause.Occur.SHOULD, *queries, **terms)
     @classmethod
     def all(cls, *queries, **terms):
-        "Return lucene BooleanQuery (AND) from queries and terms."
+        "Return :class:`BooleanQuery` (AND) from queries and terms."
         return cls.boolean(lucene.BooleanClause.Occur.MUST, *queries, **terms)
     @classmethod
     def span(cls, name, value):
-        "Return lucene SpanTermQuery."
+        "Return :class:`SpanTermQuery <SpanQuery>`."
         return SpanQuery(lucene.SpanTermQuery, lucene.Term(name, value))
     @classmethod
+    def near(cls, name, *values, **kwargs):
+        "Return :meth:`SpanNearQuery <SpanQuery.near>` from terms."
+        queries = (cls.span(name, value) for value in values)
+        return SpanQuery.near(*queries, **kwargs)
+    @classmethod
     def prefix(cls, name, value):
         "Return lucene PrefixQuery."
         return cls(lucene.PrefixQuery, lucene.Term(name, value))
     @classmethod
     def range(cls, name, start, stop, lower=True, upper=False):
-        "Return lucene ConstantScoreRangeQuery, by default with a half-open interval."
+        "Return lucene RangeQuery, by default with a half-open interval."
         base = lucene.TermRangeQuery if hasattr(lucene, 'TermRangeQuery') else lucene.ConstantScoreRangeQuery
         return cls(base, name, start, stop, lower, upper)
     @classmethod
@@ -95,6 +100,7 @@ class Query(object):
         return Query.any(other).__isub__(self)
 
 class BooleanQuery(Query):
+    "Inherited lucene BooleanQuery with sequence interface to clauses."
     def __len__(self):
         return len(self.getClauses())
     def __iter__(self):
@@ -112,6 +118,7 @@ class BooleanQuery(Query):
         return self
 
 class SpanQuery(Query):
+    "Inherited lucene SpanQuery with additional span constructors."
     def __getitem__(self, slc):
         assert slc.start is slc.step is None, 'only prefix slice supported'
         return SpanQuery(lucene.SpanFirstQuery, self, slc.stop)
@@ -120,7 +127,7 @@ class SpanQuery(Query):
     def __or__(*spans):
         return SpanQuery(lucene.SpanOrQuery, spans)
     def near(*spans, **kwargs):
-        """Return lucene SpanNearQuery.
+        """Return lucene SpanNearQuery from SpanQueries.
         
         :param slop: default 0
         :param inOrder: default True
@@ -165,6 +172,7 @@ class Filter(lucene.PythonFilter):
             setter = self.bitSet.set
             for id in itertools.ifilter(self.docIdSet.get, xrange(self.docIdSet.size())):
                 setter(id)
+    @staticmethod
     def overlap(self, other, reader=None):
         "Return intersection count of the filters."
         return int(lucene.OpenBitSet.intersectionCount(self.getDocIdSet(reader), other.getDocIdSet(reader)))
@@ -174,3 +182,37 @@ class Filter(lucene.PythonFilter):
     def getDocIdSet(self, reader=None):
         "Return cached OpenBitSet, reader is ignored."
         return self.docIdSet
+
+class Highlighter(lucene.Highlighter):
+    """Inherited lucene Filter with stored analysis options.
+    Using span scoring in lucene 2.4 is not thread-safe.
+    
+    :param query: lucene Query
+    :param analyzer: analyzer for texts
+    :param span: only highlight terms which would contribute to a hit
+    :param formatter: optional lucene Formatter or html tag name
+    :param encoder: optional lucene Encoder
+    :param field: optional field name used to match query terms
+    :param reader: optional lucene IndexReader to compute term weights
+    """
+    def __init__(self, query, analyzer, span=True, formatter=None, encoder=None, field=None, reader=None):
+        if isinstance(formatter, basestring):
+            formatter = lucene.SimpleHTMLFormatter('<{0}>'.format(formatter), '</{0}>'.format(formatter))
+        scorer = lucene.QueryScorer if (span or not hasattr(lucene, 'QueryTermScorer')) else lucene.QueryTermScorer
+        scorer = scorer(*filter(None, [query, reader, field]))
+        lucene.Highlighter.__init__(self, *filter(None, [formatter, encoder, scorer]))
+        self.query, self.analyzer, self.span, self.field = query, analyzer, span, field
+    def fragments(self, text, count=1, field=None):
+        """Return highlighted text fragments.
+        
+        :param text: text string to be searched
+        :param count: maximum number of fragments
+        :param field: optional field to use for text analysis
+        """
+        if lucene.VERSION <= '2.4.1': # memory leak in string array
+            tokens = self.analyzer.tokenStream(field, lucene.StringReader(text))
+            if self.span:
+                tokens = lucene.CachingTokenFilter(tokens)
+                self.fragmentScorer = lucene.HighlighterSpanScorer(self.query, self.field, tokens)
+            return map(unicode, self.getBestTextFragments(tokens, text, True, count))
+        return list(self.getBestFragments(self.analyzer, field, text, count))
