@@ -26,7 +26,7 @@ else:
         "Custom implementation of lucene TypeAsPayloadTokenFilter."
         def incrementToken(self):
             result = engine.TokenFilter.incrementToken(self)
-            self.payload = self.type
+            self.payload = bytes(self.type)
             return result
 
 class BaseTest(unittest.TestCase):
@@ -40,10 +40,19 @@ class TestCase(BaseTest):
     
     def testInterface(self):
         "Indexer and document interfaces."
+        self.assertRaises(TypeError, engine.IndexSearcher)
         analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else lucene.StandardAnalyzer()
         stemmer = engine.Analyzer(analyzer, lucene.PorterStemFilter, typeAsPayload)
-        assert [token.termText() if isinstance(token, lucene.Token) else token.term for token in stemmer.tokens('hello worlds')] == ['hello', 'world']
+        token = stemmer.tokens('hello').next()
+        assert token.positionIncrement == 1
+        if not isinstance(token, lucene.Token):
+            assert engine.TokenFilter(lucene.EmptyTokenStream()).payload is None
+            assert token.term == 'hello'
+            assert token.type == token.payload == '<ALPHANUM>'
+            assert token.offset == (0, 5)
         assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
+        if lucene.VERSION >= '2.9':
+            assert str(stemmer.parse('hellos', field={'body': 1.0, 'title': 2.0})) == 'body:hello title:hello^2.0'
         indexer = engine.Indexer(analyzer=stemmer)
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
@@ -126,6 +135,13 @@ class TestCase(BaseTest):
         for doc in fixture.constitution.docs():
             indexer.add(doc)
         indexer.commit()
+        searcher = engine.MultiSearcher([indexer.directory, self.tempdir])
+        assert searcher.count() == len(searcher) == 2 * len(indexer)
+        searcher = searcher.reopen()
+        assert searcher.facets(searcher, 'amendment')['amendment'] == dict.fromkeys(map(str, range(1, 28)), 2)
+        reader = searcher.indexReader
+        del searcher
+        self.assertRaises(lucene.JavaError, reader.isCurrent)
         assert len(indexer) == len(indexer.search()) == 35
         assert sorted(indexer.names()) == ['amendment', 'article', 'date', 'text']
         articles = list(indexer.terms('article'))
@@ -170,6 +186,9 @@ class TestCase(BaseTest):
         assert hit['amendment'] == '1'
         assert sorted(hit.dict()) == ['__id__', '__score__', 'amendment', 'date']
         hits = indexer.search('text:right')
+        for name in ('amendment', 'article'):
+            indexer.filters[name] = engine.Query.prefix(name, '').filter()
+        assert indexer.facets(hits.ids, 'amendment', 'article') == {'amendment': 12, 'article': 1}
         hits = indexer.search('text:people', filter=hits.ids)
         assert len(hits) == 4
         hit, = indexer.search('date:192*')
@@ -204,16 +223,16 @@ class TestCase(BaseTest):
         query -= engine.Query.term('text', 'papers')
         assert str(query[-1]) == '-text:papers'
         assert len(query) == len(list(query)) == 2
-        queries = [engine.Query.span('text', word) for word in ('persons', 'papers', 'things')]
-        count = indexer.count(queries[0])
-        near = queries[0].near(queries[1], slop=1)
-        assert indexer.count(queries[0] - near) == count
-        near = queries[0].near(queries[1] | queries[2], slop=1)
-        assert indexer.count(queries[0] - near) == count - 1
-        assert 0 < indexer.count(queries[0][:100]) < count
-        spans = dict(indexer.spans(queries[0]))
+        span = engine.Query.span('text', 'persons')
+        count = indexer.count(span)
+        near = engine.Query.near('text', 'persons', 'papers', slop=1)
+        assert indexer.count(span - near) == count
+        near = span.near(engine.Query.span('text', 'papers') | engine.Query.span('text', 'things'), slop=1)
+        assert indexer.count(span - near) == count - 1
+        assert 0 < indexer.count(span[:100]) < count
+        spans = dict(indexer.spans(span))
         assert len(spans) == count and spans == dict(indexer.docs('text', 'persons', counts=True))
-        near = queries[0].near(queries[1], slop=2)
+        near = engine.Query.near('text', 'persons', 'papers', slop=2)
         (id, positions), = indexer.spans(near, positions=True)
         assert indexer[id]['amendment'] == '4' and positions in ([(3, 6)], [(10, 13)])
         with warnings.catch_warnings(record=True) as leaks:
@@ -279,8 +298,9 @@ class TestCase(BaseTest):
         assert count > len(hits)
         assert count == indexer.count(field.range(lng[:3], lng[:3]+'~'))
         assert count > indexer.count(engine.Query.term('state', 'CA'), filter=engine.Query.term(longitude, lng).filter())
-        hits = indexer.search('zipcode:90*')
-        (field, facets), = indexer.facets(hits.ids, 'state:county').items()
+        query = engine.Query.prefix('zipcode', '90')
+        (field, facets), = indexer.facets(indexer.search(query).ids, 'state:county').items()
+        assert [facets] == indexer.facets(query.filter(), 'state:county').values()
         assert field == 'state:county'
         la, orange = sorted(filter(facets.get, facets))
         assert la == 'CA:Los Angeles' and facets[la] > 100
@@ -288,7 +308,8 @@ class TestCase(BaseTest):
     
     def testSpatial(self):
         "Spatial tile test."
-        for PointField in [engine.PointField] + [engine.numeric.PointField] * numeric:
+        for PolygonField in [engine.PolygonField] + [engine.numeric.PolygonField] * numeric:
+            PointField, = PolygonField.__bases__
             indexer = engine.Indexer(self.tempdir, 'w')
             for name, params in fixture.zipcodes.fields.items():
                 indexer.set(name, **params)
@@ -300,7 +321,7 @@ class TestCase(BaseTest):
                     indexer.add(doc, tile=[(lng, lat)], latitude=str(lat), longitude=str(lng))
                     if doc['city'] == 'Los Angeles':
                         points.append((lng, lat))
-            assert len(list(engine.PolygonField('', precision=15).items(points))) > len(points)
+            assert len(list(PolygonField('', precision=15).items(points))) > len(points)
             indexer.commit()
             city, zipcode, tile = 'Beverly Hills', '90210', '023012311120332'
             hit, = indexer.search('zipcode:' + zipcode)
@@ -421,6 +442,7 @@ class TestCase(BaseTest):
     def testHighlighting(self):
         "Highlighting text fragments."
         indexer = engine.Indexer()
+        indexer.set('text', store=True)
         amendments = dict((doc['amendment'], doc['text']) for doc in fixture.constitution.docs() if 'amendment' in doc)
         for amendment in amendments.values():
             fragments = indexer.highlight('persons', amendment)
@@ -433,8 +455,12 @@ class TestCase(BaseTest):
         assert len(fragments) == 2 and fragments[0].count('*') == 2*3 and '*persons*' in fragments[1]
         fragment, = indexer.highlight(query, text, count=3, span=False, textFragmenter=lucene.SimpleFragmenter(200))
         assert len(fragment) > len(text) and fragment.count('<B>persons</B>') == 2
-        fragment, = indexer.highlight(query, text, count=3)
-        assert len(fragment) < len(text) and fragment.count('<B>') == 3
+        fragment, = indexer.highlight(query, text, count=3, formatter='em')
+        assert len(fragment) < len(text) and fragment.count('<em>') == 3
+        indexer.add(text=text)
+        indexer.commit()
+        fragment, = indexer.highlight(query, 0, field='text')
+        assert fragment.count('<B>') == fragment.count('</B>') == 3
 
 if __name__ == '__main__':
     lucene.initVM(lucene.CLASSPATH)
